#include "xuartpsv.h"

XUartPsv_Config XUartPsv_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"arm,pl011", /* compatible */
		0xff000000, /* reg */
		0x5f5dd19, /* xlnx,clock-freq */
		0x0, /* ModemPinsConnected */
		0x1c200, /* xlnx,baudrate */
		0x4012, /* interrupts */
		0xf9000000 /* interrupt-parent */
	},
	{
		"arm,pl011", /* compatible */
		0xff010000, /* reg */
		0x5f5dd19, /* xlnx,clock-freq */
		0x0, /* ModemPinsConnected */
		0x1c200, /* xlnx,baudrate */
		0x4013, /* interrupts */
		0xf9000000 /* interrupt-parent */
	},
	 {
		 NULL
	}
};